#include <Wire.h>
#include "AM2320Display.h"

// 温湿度センサー AM2320 DEMO

#define BAUDRATE 9600
#define AM2320ADDR 0x5C
#define SWITCHPIN 5

AM2320Display display ;

char str[120] ;
bool graphMode = false;

// ----- Senser Data
struct AM2320_s {
  uint8_t humidityH ;   // 湿度の HIGH Byte
  uint8_t humidityL ;   // 湿度の LOW BYTE
  uint8_t temperatureH; // 温度の HIGH Byte
  uint8_t temperatureL; // 温度の LOW Byte
} am2320 ;

bool readAM2320( ) {
  uint8_t readData[8] ;
  // wake up
  Wire.beginTransmission(AM2320ADDR);
  Wire.write(0x00);
  Wire.endTransmission();
  delay(10); // wait 10 ms

  Wire.beginTransmission(AM2320ADDR);
  Wire.write(0x03);
  Wire.write(0x00);
  Wire.write(0x04);
  Wire.endTransmission();
  delay(2); // wait 2 ms

  // 4Byte + CRC Read
  Wire.requestFrom(AM2320ADDR, 8);
  int len = 0 ;
  while( len < 8) {
    if (Wire.available() > 0) {
      readData[len] = Wire.read();
      len ++ ;
    }
  }

  // ----- CRC16 -----
  int readLen = 6 ;
  uint16_t rCrc = readData[7] << 8 | readData[6] ; 
  uint16_t crc = 0xffff;
  for (int i = 0; i < readLen ; i++) {
    uint8_t b = readData[i];
    crc ^= b;
    for (int x = 0; x < 8; x++) {
      if (crc & 0x0001) {
        crc >>= 1;
        crc ^= 0xA001;
      } else {
        crc >>= 1;
      }
    }
  }

  // ----- 取得データ格納 -----
  am2320.humidityH = readData[2] ;   // 湿度の HIGH Byte
  am2320.humidityL = readData[3] ;   // 湿度の LOW Byte
  am2320.temperatureH = readData[4]; // 温度の HIGH Byte
  am2320.temperatureL = readData[5]; // 温度の LOW Byte
  return (rCrc == crc) ;
}

// ----------------------
// ----- セットアップ -----
// ----------------------
void setup() {
  Serial.begin(BAUDRATE);
  Serial.println("\nDHT11 - Read") ;

  Wire.begin();
  display.init() ;
}

// ----------------------
// ----- ループ-----
// ----------------------
void loop() {
  if (digitalRead(SWITCHPIN)) {
    graphMode = !graphMode ;
    display.cls( ) ;
  }

  // ----- 温湿度読み取り -----
  if (readAM2320( )) {
    int hum = (am2320.humidityH << 8) + am2320.humidityL ;
    int tem = ((am2320.temperatureH & 0x7F) << 8) + am2320.temperatureL ;
    if ((am2320.temperatureH & 0x80) != 0) tem *= -1 ;
    sprintf(str,"湿度:%2d.%d%%  気温:%2d.%d℃", hum / 10 , hum % 10 , tem / 10 , tem % 10) ;
    Serial.println(str) ;

    display.addGraphData(hum / 10 , tem / 10) ;

    if (graphMode) {
      // --- Graph表示
      display.dispGraphData();
    } else {    
      // --- 7セグメントLEDっぽい表示 ---
      display.disp7segLED(hum,tem) ;
    }
    delay(5000) ;
  } else {
    // 読み取り失敗：2秒 Wait
    delay(2000) ;
  }
}
